-- GLOBAL --
SAM = select(2, ...); 
-- MODULE --
local _G = getfenv(0);
setmetatable(SAM, {__index = _G});
setfenv(1, SAM);
-- LOCAL --
title		= "StopAddonMessage"
folder		= title;
version		= GetAddOnMetadata(folder, "X-Curse-Packaged-Version") or "";
titleFull	= title.." "..version

coreFrame = CreateFrame("Frame", nil, UIParent);
core	= LibStub("AceAddon-3.0"):NewAddon(coreFrame, title, "AceConsole-3.0", "AceHook-3.0") -- 
local L = LibStub("AceLocale-3.0"):GetLocale(title, true)
P = {}; --db.profile

lastOnUpdate = 0;

directory	= "Interface\\Addons\\"..folder.."\\";

--Globals
local GAMEOPTIONS_MENU_ = _G.GAMEOPTIONS_MENU;
local ENABLE_ = _G.ENABLE;
local pairs_ = _G.pairs;
local InterfaceOptionsFrame_OpenToCategory_ = _G.InterfaceOptionsFrame_OpenToCategory;
local tostring_ = _G.tostring;
local table_getn = _G.table.getn;
local print_ = _G.print;


-- Default Settings
defaultSettings = {profile = {}}
defaultSettings.profile.defaultAction	= 0;--0=allow, 1=deny
defaultSettings.profile.printMessages		= 4;--print new messages
defaultSettings.profile.outgoingChatFrame	= 1;--main chatframe.

defaultSettings.profile.prefixActions	= {};

-- Echo Strings
local strDebugFrom		= "|cffffff00[%s]|r" --Yellow function name. help pinpoint where the debug msg is from.
local strWhiteBar		= "|cffffff00 || |r" -- a white bar to seperate the debug info.
local greenText			= "|cff00ff00%s|r";
local redText			= "|cffff0000%s|r"

local coreUI;
local prefixUI;
local newPrefixes = {};


-- Ace3 Functions
function core:OnInitialize()
	db = LibStub("AceDB-3.0"):New("SAM_DB", defaultSettings)
	CoreOptionsTable.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)--save option profile or load another chars opts.
	db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")

	P = db.profile;

	core:RegisterChatCommand("sam", "MySlashProcessorFunc")
	
	
	BuildPrefixUIs()
	local config = LibStub("AceConfig-3.0");
	local dialog = LibStub("AceConfigDialog-3.0");
	config:RegisterOptionsTable(title, CoreOptionsTable)
	coreUI = dialog:AddToBlizOptions(title, "SAM "..version)
	
	config:RegisterOptionsTable(title.."Prefix", PrefixOptionsTable)
	prefixUI = dialog:AddToBlizOptions(title.."Prefix", "Prefix", "SAM "..version)

end

function core:OnEnable()
	--make all options clickable.
	for name, data in pairs_(CoreOptionsTable.args.core.args) do 
		if name ~= "enable" then
			CoreOptionsTable.args.core.args[name].disabled = false;
		end
	end
	
	self:RawHook("SendAddonMessage", true)
	BuildPrefixUIs()
end

----------------------------------------------
function core:MySlashProcessorFunc(input)	--
-- /att function brings up the UI options.	--
----------------------------------------------
	InterfaceOptionsFrame_OpenToCategory_(prefixUI)
	InterfaceOptionsFrame_OpenToCategory_(coreUI)
end

--	/script SendAddonMessage("123", "blaw","GUILD")

--	/script SAM.P.prefixActions = {}

function core:SendAddonMessage(...)
	local prefix, text, type, target = ...;
--~ 	print("SAM","SendAddonMessage",...)
	
	if P.prefixActions[prefix] then
		if P.prefixActions[prefix].action == 0 then --allow
			P.prefixActions[prefix].allowed = P.prefixActions[prefix].allowed + 1;
			
			PrintAddonMessage(L["Allowed"], false, ...)
		else
			P.prefixActions[prefix].denied = P.prefixActions[prefix].denied + 1;
			PrintAddonMessage(L["Denied"], false, ...)
			return;
		end
	else
		newPrefixes[prefix] = true;
		P.prefixActions[prefix] = {denied=0,allowed=0}
		if P.defaultAction == 0 then
			P.prefixActions[prefix].action = 0;
			P.prefixActions[prefix].allowed = 1;

			PrintAddonMessage(L["Allowed"], true, ...)
			
			BuildPrefixUIs()
		else
			P.prefixActions[prefix].action = 1;
			P.prefixActions[prefix].denied = 1;


			PrintAddonMessage(L["Denied"], true, ...)
			
			BuildPrefixUIs()
			return;
		end
	end
	
	core.hooks.SendAddonMessage(...)
end

--	/script SAM.P.prefixActions = {};

function PrintAddonMessage(action, new, ...)
	local prefix, message, distribution, target  = ...;
	
	if not P.prefixActions[prefix].printMsgs or P.prefixActions[prefix].printMsgs == false then
		if P.printMessages == 0 then
			return;
		elseif P.printMessages == 2 and action ~= L["Allowed"] then
			return;
		elseif P.printMessages == 3 and action ~= L["Denied"] then
			return;
		elseif P.printMessages == 4 and new ~= true then
			return;
		end
	end
	
	local cf = _G["ChatFrame"..P.outgoingChatFrame];
	if cf then
		if distribution == "WHISPER" then
			cf:AddMessage("|cffffffff"..action.."|r> |cffff0000"..prefix.."|r || |cff00ff00"..distribution.."|r || |cff00ffff"..target.."|r || "..message,.5,.5,.5)
		else
			cf:AddMessage("|cffffffff"..action.."|r> |cffff0000"..prefix.."|r || |cff00ff00"..distribution.."|r || "..message,.5,.5,.5)
		end
	end
end


function core:OnDisable()
	--make all the options unclickable except for the enable button.
	for name, data in pairs_(CoreOptionsTable.args.core.args) do 
		if name ~= "enable" then
			CoreOptionsTable.args.core.args[name].disabled = true;
		end
	end
end

function BuildPrefixUIs()
	PrefixOptionsTable.args = {}; --reset

	local list = {};
	for prefix, data in pairs(P.prefixActions) do 
		table.insert(list, prefix)
	end
	
	table.sort(list, function(a,b) 
		if(a and b) then 
			return a < b;
		end 
	end)
	
	local prefix;
	local pColoured;
	for i=1, table.getn(list) do 
		prefix = list[i];
		
		if P.prefixActions[prefix].action == 0 then
			pColoured = greenText:format(prefix);
		else
			pColoured = redText:format(prefix);
		end
		
		if newPrefixes[prefix] then
			pColoured = pColoured .. " |cffffff00+|r";--yellow + symble.
		end
		
		
		PrefixOptionsTable.args[prefix] = {
			type	= "group",
			name	= pColoured,
			desc	= (knownPrefixes[prefix] or L["Unknown"]),
			order = i,
			args={}
		}
		
		
		PrefixOptionsTable.args[prefix].args.addon = {
			type = "description",
			name = L["Addon: "]..(knownPrefixes[prefix] or L["Unknown"]),
			order = 1,
		}

		PrefixOptionsTable.args[prefix].args.action = {
			name = L["Action"],
			order = 4,
			desc = L["What to do when message is sent."],
			type = "select",
			
			values = function()
				return {[0]=greenText:format(L["Allow"]), [1]=redText:format(L["Deny"])}
			end,
			set = function(info,val) 
				P.prefixActions[info[1]].action = val;
				BuildPrefixUIs()
			end,
			get = function(info) return P.prefixActions[info[1]].action end
		}

		PrefixOptionsTable.args[prefix].args.allowed = {
			type = "description",
			name = L["Allowed"]..": "..P.prefixActions[prefix].allowed,
			order = 5,
		}
		
		PrefixOptionsTable.args[prefix].args.denied = {
			type = "description",
			name = L["Denied"]..": "..P.prefixActions[prefix].denied,
			order = 6,
		}

		PrefixOptionsTable.args[prefix].args.printMsgs = {
			type = "toggle",	order	= 7,
			name	= L["Print Msg"],
			desc	= L["Print this prefix's messages to chat frame."],
			set = function(info,val) 
				P.prefixActions[info[1]].printMsgs = val;
			end,
			get = function(info) return P.prefixActions[info[1]].printMsgs end
		}	
		
		
	end
	
end

----------------------------------------------------------------------
function core:OnProfileChanged(...)									--
-- User has reset proflie, so we reset addon.						--
----------------------------------------------------------------------
	-- Shut down anything left from previous settings
	self:Disable()
	-- Enable again with the new settings
	self:Enable()
end



------------------------------
function Debug(from, ...)	--	/script DA.Debug("hey","bye")
-- simple print function.	--
------------------------------
	if version ~= "" then--version's from curse, user doesn't need to see debug msgs.
		return;
	end
	local tbl  = {...}
	local msg = tostring_(tbl[1])
	for i=2,table_getn(tbl) do 
		msg = msg..strWhiteBar..tostring_(tbl[i]);
	end
	print_(strDebugFrom:format(from).." "..msg)
end


----------------
--     UI     --
----------------

PrefixOptionsTable = {
	name = titleFull,
	type = "group",
	args = {},
}

CoreOptionsTable = {
	name = titleFull,
	type = "group",
	childGroups = "tab",

	args = {
		core={
			name = GAMEOPTIONS_MENU_,
			type = "group",
			order = 1,
			args={}
		},
	},
}




CoreOptionsTable.args.core.args.enable = {
	type = "toggle",	order	= 1,
	name	= ENABLE_,
	desc	= L["Enables / Disables the addon"],
	set = function(info,val) 
		if val == true then
			core:Enable();
		else
			core:Disable();
		end
	end,
	get = function(info) return core:IsEnabled() end
}

CoreOptionsTable.args.core.args.defaultAction = {
	name = L["Default Action"],
	order = 3,
	desc = L["Default action for new prefixes"],
	type = "select",
	
	values = function()
		return {[0]=greenText:format(L["Allow"]), [1]=redText:format(L["Deny"])}
	end,
	set = function(info,val) 
		P.defaultAction = val;
	end,
	get = function(info) return P.defaultAction end
}



CoreOptionsTable.args.core.args.printMessages = {
	name = L["Print Messages"],
	order = 4,
	desc = L["Print your outgoing messages to a chatframe."],
	type = "select",
	
	values = function()
		return {
			[0] = L["Off"],
			[1] = L["All"],
			[2] = L["Allowed"],
			[3] = L["Denied"],
			[4] = L["New"],
		}
		
	end,
	set = function(info,val) 
		P.printMessages = val;
	end,
	get = function(info) return P.printMessages end
}


CoreOptionsTable.args.core.args.outgoingChatFrame = {
	name = L["Chat Frame"],
	order = 5,
	desc = L["Chatframe to print messages to"],
	type = "select",
	
	values = function()
		local tbl = {};
		
		for i=1, 10 do 
			table.insert(tbl,(_G["ChatFrame"..i] and _G["ChatFrame"..i].name) or "ChatFrame"..i)
		end
	
		return tbl
		
	end,
	set = function(info,val) 
		P.outgoingChatFrame = val;
	end,
	get = function(info) return P.outgoingChatFrame end
}